@@ -1,15 +1,20 @@
 import json
 import logging
 import os
+import re
 import sys
+from functools import partial
 
 from asgi_correlation_id import CorrelationIdMiddleware
-from asgi_correlation_id.context import correlation_id
 from fastapi import FastAPI
 from loguru import logger
 from starlette.middleware.base import BaseHTTPMiddleware
 
-from .middleware import http_request_context, http_request_middleware_func
+from .middleware import (
+    cloud_trace_context,
+    http_request_context,
+    http_request_middleware_func,
+)
 
 
 class InterceptHandler(logging.Handler):
@@ -36,9 +41,14 @@ class InterceptHandler(logging.Handler):
         )
 
 
-def stackdriver_sink(message):
+def stackdriver_sink(message, project):
+    """
+    Uses google's structured logging to get nice logs in stackdriver.
+    https://cloud.google.com/logging/docs/structured-logging for more info.
+    This guide has also server as inspiration
+    https://dev.to/floflock/enable-feature-rich-logging-for-fastapi-on-google-cloud-logging-j3i
+    """
     record = message.record
-    corr_id = correlation_id.get()
     http_request = http_request_context.get()
     log_info = {
         "severity": record["level"].name,
@@ -53,16 +63,27 @@ def stackdriver_sink(message):
     if http_request is not None:
         log_info["httpRequest"] = http_request
 
-    if corr_id is not None:
+    trace = cloud_trace_context.get()
+    if trace is not None:
+        split_header = trace.split("/", 1) if trace else None
+
+        trace_id = f"projects/{project}/traces/{split_header[0]}"
+
+        header_suffix = split_header[1]
+        span_id = re.findall(r"^\w+", header_suffix)[0]
+
         log_info[
             "logging.googleapis.com/trace"
-        ] = f"projects/databutton/traces/{correlation_id.get()}"
-
+        ] = f"projects/databutton/traces/{trace_id}"
+        log_info["spanId"] = span_id
     serialized = json.dumps(log_info)
     print(serialized, file=sys.stderr)
 
 
-def setup_logging_fastapi_gcp(app: FastAPI):
+def setup_logging_fastapi_gcp(app: FastAPI, GCP_PROJECT="databutton"):
+    """
+    Sets up logging and tracing for FastAPI based projects.
+    """
     app.add_middleware(BaseHTTPMiddleware, dispatch=http_request_middleware_func)
     app.add_middleware(CorrelationIdMiddleware)
 
@@ -79,8 +100,9 @@ def setup_logging_fastapi_gcp(app: FastAPI):
     logging.getLogger("uvicorn").handlers = [intercept_handler]
 
     logger.remove()
+    sink_for_project = partial(stackdriver_sink, project=GCP_PROJECT)
     logger.add(
-        stackdriver_sink,
+        sink_for_project,
         serialize=True,
         level=os.environ.get("LOGURU_LEVEL", "INFO"),
     )
